@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2023 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -36,6 +36,7 @@
   #:use-module (dzn code language dzn)
   #:use-module (dzn code language c++)
   #:use-module (dzn command-line)
+  #:use-module (dzn lts)
   #:use-module (dzn misc)
   #:export (c++:call-check-bindings
             c++:event-method
@@ -485,7 +486,7 @@ std::basic_ostream<Char, Traits> &")
                 else)))
 
     (define (value->init value)
-      (simple-format #f "~s" (.value value)))
+      (simple-format #f "~s" value))
 
     (define (dzn:hash r h)
       (let ((overflow 4294967296)
@@ -511,15 +512,14 @@ std::basic_ostream<Char, Traits> &")
 
 
     (define (event->sm:switch-case transition interface-shared)
-      (let* ((prefix (.prefix transition))
-             (prefix (ast:statement* prefix))
+      (let* ((prefix (edge-label transition))
              (initializer-list (sm:code->string
                                 (sm:generalized-initializer-list*
-                                 (map value->init prefix))))
-             (state (simple-format #f "~a" (.to transition)))
+                                 `(,(value->init prefix)))))
+             (state (simple-format #f "~a" (edge-to transition)))
              (assignments (and=>
                            (find
-                            (compose (cute eq? (.to transition) <>)
+                            (compose (cute eq? (edge-to transition) <>)
                                      .state)
                             interface-shared)
                            (compose ast:statement* .assign)))
@@ -527,23 +527,33 @@ std::basic_ostream<Char, Traits> &")
         (sm:switch-case
          (expression (string-append
                       (number->string
-                       (dzn:hash (map .value prefix)
-                                 (.from transition)))
+                       (dzn:hash `(,prefix)
+                                 (edge-from transition)))
                       "u"))
          (statement
           (sm:statements*
            (cons (sm:comment* (string-append
-                               "//" (number->string (.from transition))
-                               ":" (string-join (map .value prefix) ",")))
-                 (if (not (.rtc? transition)) `(,(sm:return*))
-                     `(,(sm:assign* "dzn_state" state)
-                       ,@assignments
-                       ,(sm:break)))))))))
+                               "//" (number->string (edge-from transition))
+                               ":" prefix))
+                 `(,(sm:assign* "dzn_state" state)
+                   ,@assignments
+                   ,(sm:break))))))))
 
     (define (event->transitions event)
       (let* ((transitions (code:shared event)))
         transitions))
 
+    (define (transition< a b)
+      (or (< (edge-from a) (edge-from b))
+          (and (= (edge-from a) (edge-from b))
+               (string<? (edge-label a)
+                         (edge-label b)))))
+
+    (define (transition= a b)
+      (and
+       (= (edge-from a) (edge-from b))
+       (string=? (edge-label a) (edge-label b))))
+
     (let* ((type (string-join (cons "" (ast:full-name o)) "::"))
            (type& (string-append type "&"))
            (connect
@@ -608,7 +618,7 @@ std::basic_ostream<Char, Traits> &")
                                                         event-name ".set"))
                                    (arguments `("that", "this",
                                                 event-name-string)))))
-                          (ast:event* o)))))
+                      (ast:event* o)))))
                  ,(sm:destructor (struct interface)
                                  (type "virtual")
                                  (statement "= default;"))
@@ -668,8 +678,10 @@ std::basic_ostream<Char, Traits> &")
                               (events (ast:event* o))
                               (transitions (append-map event->transitions
                                                        events))
-                              (transitions (delete-duplicates transitions
-                                                              code:prefix-equal?)))
+                              (transitions (sort transitions transition<))
+                              (transitions (delete-adjacent-duplicates
+                                            transitions
+                                            transition=)))
                          `(,(sm:method
                              (struct interface) (type "void")
                              (name "dzn_event")
@@ -1156,19 +1168,6 @@ std::basic_ostream<Char, Traits> &")
 (define-method (shell-system->sm:statements-unmemoized (o <shell-system>))
   (let* ((injected-instances (code:injected-instance* o))
          (injected? (pair? injected-instances)))
-    (define (port->external port)
-      (map
-       (lambda (event)
-         (sm:assign*
-          (simple-format
-           #f "~a.~a.~a.dzn_shell_p"
-           (.name port)
-           (.direction event)
-           (.name event))
-          "true"))
-       (append
-        (ast:in-event* port)
-        (ast:out-event* port))))
     (define (provides->member port)
       (let ((other-end (ast:other-end-point port)))
         (sm:variable
@@ -1346,9 +1345,7 @@ std::basic_ostream<Char, Traits> &")
                                            (name "locator"))))
                  (statement
                   (sm:compound*
-                   `(,@(append-map port->external (ast:provides-port* o))
-                     ,@(append-map port->external (ast:requires-port* o))
-                     ,@(map provides->init (ast:provides-port* o))
+                   `(,@(map provides->init (ast:provides-port* o))
                      ,@(map requires->init (ast:requires-port* o))
                      ,@(map trigger->event-slot (ast:provides-in-triggers o))
                      ,@(map trigger->event-slot (ast:requires-out-triggers o))
