@@ -34,7 +34,7 @@
   #:use-module (dzn ast goops)
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn code makreel)
+  #:use-module (dzn code language makreel)
   #:use-module (dzn command-line)
   #:use-module (dzn commands parse)
   #:use-module (dzn config)
@@ -53,7 +53,6 @@
             (help (single-char #\h))
             (illegal (single-char #\i))
             (import (single-char #\I) (value #t))
-            (jitty (single-char #\j))
             (lts (single-char #\l))
             (traces (single-char #\t))
             (model (single-char #\m) (value #t))
@@ -63,10 +62,10 @@
             (queue-size (single-char #\q) (value #t))
             (queue-size-defer (value #t))
             (queue-size-external (value #t))))
-	 (options (getopt-long args option-spec))
-	 (help? (option-ref options 'help #f))
-	 (files (option-ref options '() '()))
-	 (usage? (and (not help?) (null? files))))
+         (options (getopt-long args option-spec))
+         (help? (option-ref options 'help #f))
+         (files (option-ref options '() '()))
+         (usage? (and (not help?) (null? files))))
     (when (or help? usage?)
       (let ((port (if usage? (current-error-port) (current-output-port))))
         (format port "\
@@ -78,7 +77,6 @@ Generate exhaustive set of traces for Dezyne model
   -f, --flush                 include <flush> events in trace
   -h, --help                  display this help and exit
   -i, --illegal               include traces that lead to an illegal
-  -j, --jitty                 pass to lps2lts, for LARGE models
   -I, --import=DIR+           add DIR to import path
   -l, --lts                   also generate LTS
   -m, --model=MODEL           generate traces for model MODEL
@@ -99,13 +97,12 @@ Generate exhaustive set of traces for Dezyne model
          (text (regexp-substitute/global #f "\"[^\"]*\\.qout\\.[^\"]*\"" text 'pre "\"tau\"" 'post))
          (text (regexp-substitute/global #f "\"(optional|inevitable)\"" text 'pre "\"tau\"" 'post))
          (text (regexp-substitute/global #f "\"[^\"]*\\.(optional|inevitable)\"" text 'pre "\"tau\"" 'post))
+         (text (regexp-substitute/global #f "\"[^\"]*<state>[^\"]*\"" text 'pre "\"tau\"" 'post))
          (text (regexp-substitute/global #f "\"tag[()].[^\"]*\"" text 'pre "\"tau\"" 'post)))
     text))
 
 (define (model->lts root model file-name)
-  (let* ((jitty? (command-line:get 'jitty))
-         (lts (verify-pipeline (if jitty? "aut-weak-trace-jitty"
-                                   "aut-weak-trace")
+  (let* ((lts (verify-pipeline "aut-weak-trace"
                                root model))
          (lts (lts-hide-internal-labels lts)))
     (when (string-null? (string-trim-right lts))
@@ -153,7 +150,6 @@ Generate exhaustive set of traces for Dezyne model
   (let* ((options (parse-opts args))
          (files (option-ref options '() '()))
          (file-name (car files))
-         (ast (parse options file-name))
          (model-name (option-ref options 'model #f))
          (queue-size (option-ref options 'queue-size (%queue-size)))
          (queue-size-defer (option-ref options 'queue-size-defer
@@ -162,12 +158,14 @@ Generate exhaustive set of traces for Dezyne model
                                           (%queue-size-external))))
     (define (named? o)
       (equal? (makreel:unticked-dotted-name o) model-name))
-    (parameterize ((%no-unreachable? #t)
+    (parameterize ((%language "makreel")
+                   (%no-unreachable? #t)
                    (%queue-size queue-size)
                    (%queue-size-defer queue-size-defer)
                    (%queue-size-external queue-size-external))
-      (let* ((root (makreel:normalize ast))
-             (models (ast:model* root))
+      (let* ((ast (parse options file-name))
+             (root (makreel:normalize ast))
+             (models (ast:model** root))
              (components-interfaces
               (append
                (filter (conjoin (is? <component>) .behavior) models)
