@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2020, 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2018, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018 Johri van Eerd <vaneerd.johri@gmail.com>
@@ -34,7 +34,7 @@
 
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn code makreel)
+  #:use-module (dzn code language makreel)
   #:use-module (dzn command-line)
   #:use-module (dzn commands parse)
   #:use-module (dzn config)
@@ -48,7 +48,6 @@
           '((all (single-char #\a))
             (help (single-char #\h))
             (import (single-char #\I) (value #t))
-            (jitty (single-char #\j))
             (model (single-char #\m) (value #t))
             (no-constraint (single-char #\C))
             (no-interfaces)
@@ -58,11 +57,11 @@
             (queue-size (single-char #\q) (value #t))
             (queue-size-defer (value #t))
             (queue-size-external (value #t))))
-	 (options (getopt-long args option-spec))
-	 (help? (option-ref options 'help #f))
-	 (files (option-ref options '() '()))
+         (options (getopt-long args option-spec))
+         (help? (option-ref options 'help #f))
+         (files (option-ref options '() '()))
          (out (option-ref options 'out #f))
-	 (usage? (and (not help?) (null? files))))
+         (usage? (and (not help?) (null? files))))
     (when (equal? out "help")
       (format #t "formats:~a\n" (string-join (verification:formats) "\n  " 'prefix))
       (exit EXIT_SUCCESS))
@@ -77,7 +76,6 @@ Check DZN-FILE for verification errors in Dezyne models
   -D, --no-non-compliance  report deadlock upon non-compliance
   -h, --help               display this help and exit
   -I, --import=DIR+        add DIR to import path
-  -j, --jitty              run lps2lts with --rewriter=jittyc
   -m, --model=MODEL        restrict verification to model MODEL
       --no-interfaces      skip interface verification
       --out=FORMAT         produce output FORMAT (use \"help\" for a list)
@@ -88,7 +86,7 @@ Check DZN-FILE for verification errors in Dezyne models
       --queue-size-external=SIZE
                            use external queue size=SIZE for verification [~a]
 " (%queue-size) (%queue-size-defer) (%queue-size-external))
-	(exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
+        (exit (or (and usage? EXIT_OTHER_FAILURE) EXIT_SUCCESS))))
     options))
 
 (define (main args)
@@ -100,22 +98,26 @@ Check DZN-FILE for verification errors in Dezyne models
          (debug? (dzn:command-line:get 'debug #f))
          (out (option-ref options 'out #f))
          (model-name (option-ref options 'model #f))
-         (ast (parse options file-name))
-         (model (and model-name (call-with-handle-exceptions
-                                 (lambda _ (ast:get-model ast model-name))
-                                 #:backtrace? debug?
-                                 #:file-name file-name)))
          (no-unreachable? (command-line:get 'no-unreachable))
+         (no-constraint? (command-line:get 'no-constraint))
          (queue-size (option-ref options 'queue-size (%queue-size)))
          (queue-size-defer (option-ref options 'queue-size-defer
                                        (%queue-size-defer)))
          (queue-size-external (option-ref options 'queue-size-external
                                           (%queue-size-external))))
-    (parameterize ((%no-unreachable? no-unreachable?)
+    (parameterize ((%language "makreel")
+                   (%no-constraint? no-constraint?)
+                   (%no-unreachable? no-unreachable?)
                    (%queue-size queue-size)
                    (%queue-size-defer queue-size-defer)
                    (%queue-size-external queue-size-external))
-      (let ((root (makreel:normalize ast)))
+
+      (let* ((ast (parse options file-name))
+             (model (and model-name (call-with-handle-exceptions
+                                     (lambda _ (ast:get-model ast model-name))
+                                     #:backtrace? debug?
+                                     #:file-name file-name)))
+             (root (makreel:normalize ast)))
         (when (and=> model ast:imported?)
           (let ((name (ast:dotted-name model)))
             (format (current-error-port)
