@@ -1,9 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019, 202, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2014, 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2014, 2018, 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2023 Karol Kobiela <karol.kobiela@verum.com>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -38,20 +39,25 @@
   #:use-module (dzn ast display)
   #:use-module (dzn ast goops)
   #:use-module (dzn ast parse)
+  #:use-module (dzn ast silence)
   #:use-module (dzn ast)
   #:use-module (dzn command-line)
+  #:use-module (dzn misc)
   #:use-module (dzn parse peg)
-  #:use-module (dzn parse peg)
+  #:use-module (dzn parse tree)
   #:use-module (dzn ast wfc)
 
+  #:declarative? #f
+
   #:export (file->ast
             file->stream
             file+import-content-alist
-            parse:handle-exceptions
             call-with-handle-exceptions
             peg:handle-syntax-error
             string->ast
-            string->parse-tree))
+            string->parse-tree
+            string->fall-back-parse-tree
+            format-display-syntax-error))
 
 (define (peg:line-number string pos)
   (1+ (string-count string #\newline 0 pos)))
@@ -92,13 +98,18 @@
   (display (peg:imported-from->message content-alist file-name) (current-error-port))
   (display (peg:message file-name string pos "error" message) (current-error-port)))
 
-(define (peg:syntax-error->message e)
+(define (peg:syntax-error->message e string pos)
   (define (unknown-identifier? e)
     (match e
       (('not-followed-by 'unknown-identifier) #t)
       ((symbol items ...) (any unknown-identifier? items))
       (_ #f)))
 
+  (define (name string pos)
+    (let ((m (string-match "[a-zA-Z_][a-zA-Z_0-9]*" (substring string pos))))
+      (or (match:substring m)
+          "")))
+
   (define (error->string e)
     (match e
       ('port-trigger "trigger")
@@ -108,7 +119,7 @@
       ('dq-string "double quoted string")
       ('compound-name "name or dotted name")
       ('BRACE-OPEN "{")
-      ('BRACE-OPEN "}")
+      ('BRACE-CLOSE "}")
       ('SEMICOLON ";")
       ('COMMA "comma")
       ('COLON ":")
@@ -125,14 +136,14 @@
       ((? string?) e)))
 
   (let ((message (error->string e)))
-    (if (not (unknown-identifier? e))  message
-        (string-append "unknown identifier; " message))))
+    (if (not (unknown-identifier? e)) (string-append "`" message "' expected")
+        (string-append "undefined identifier `" (name string pos) "'"))))
 
 (define* (peg:handle-syntax-error file-name string #:key (content-alist '()))
   (lambda (key . args)
     (unless (or (null? args) (null? (car args)))
       (let* ((pos (caar args))
-             (message (format #f "`~a' expected" (peg:syntax-error->message (cadar args)))))
+             (message (peg:syntax-error->message (cadar args) string pos)))
         (peg:error-message content-alist file-name string pos message)))
     (apply throw key args)))
 
@@ -156,6 +167,21 @@ such unnamed lists."
        tree)))
   (map helper tree))
 
+(define (format-capture-syntax-error error-collector)
+  "Return a procedure that generates a human-readable message and passes
+it to the ERROR-COLLECTOR procedure."
+  (lambda (str line-number col-number error-type error)
+    (let ((message (peg:syntax-error->message (cadar error) str (caar error))))
+      (error-collector error-type message line-number col-number))))
+
+(define (format-display-syntax-error file-name)
+  "Return a procedure that to format GNU style error message for
+FILE-NAME."
+  (lambda (str line-number col-number error-type error)
+    (let ((message (peg:syntax-error->message (cadar error) str (caar error))))
+      (format (current-error-port) "~a:~a:~a: ~a\n"
+              file-name line-number col-number message))))
+
 (define* (string->parse-tree string #:key (file-name "-") (content-alist '()))
   (let ((fall-back? (%peg:fall-back?)))
     (define (parse)
@@ -167,7 +193,8 @@ such unnamed lists."
                                        `((file-name ,file-name (location 0 0))))
                                ,@tree))))
              (parse-tree (if (%peg:fall-back?) (peg:flatten-tree parse-tree)
-                             parse-tree)))
+                             parse-tree))
+             (parse-tree (tree:normalize parse-tree)))
         (when (> (dzn:debugity) 2)
           (format (current-error-port) "parse-tree: ~a\n" file-name)
           (pretty-print parse-tree (current-error-port)))
@@ -176,12 +203,24 @@ such unnamed lists."
                    (%peg:locations? #t)
                    (%peg:skip? peg:skip-parse)
                    (%peg:debug? (> (dzn:debugity) 3)))
-      ;; in case of fall-back, try a regular parse first
+      ;; In case of fall-back, try a regular parse first
       (catch 'syntax-error
-        parse
+        (lambda () (values (parse) #f))
         (lambda (key . args)
-          (if fall-back? (parameterize ((%peg:fall-back? #t)) (parse))
-              (apply (peg:handle-syntax-error file-name string #:content-alist content-alist) key args)))))))
+          (if fall-back? (parameterize ((%peg:fall-back? #t))
+                           (values (parse) #t))
+              (apply (peg:handle-syntax-error file-name string
+                                              #:content-alist content-alist)
+                     key args)))))))
+
+(define (string->fall-back-parse-tree text content-alist
+                                      file-name error-collector)
+  "Wrapper for running string->parse-tree in fall-back mode only."
+  (parameterize ((%peg:fall-back? #t)
+                 (%peg:error (format-capture-syntax-error error-collector)))
+    (string->parse-tree text
+                        #:content-alist content-alist
+                        #:file-name file-name)))
 
 (define (parse-file+import-content-alist alist)
   "From ALIST of form
@@ -189,18 +228,28 @@ such unnamed lists."
    '((FILE-NAME . CONTENT)
      (IMPORTED-FILE-NAME . IMPORTED-CONTENT) ...)
 
-parse CONTENT and return
+parse CONTENT and return multiple values
 
-   '((FILE-NAME . PARSE-TREE)
-     (IMPORTED-FILE-NAME . IMPORTED-PARSE-TREE) ...)
-"
-  (map (match-lambda
-         ((file-name . content)
-          (cons file-name (string->parse-tree
-                           content
-                           #:file-name file-name
-                           #:content-alist alist))))
-       alist))
+'((FILE-NAME . PARSE-TREE)
+  (IMPORTED-FILE-NAME . IMPORTED-PARSE-TREE) ...)
+
++
+
+  #true if parsing of any CONTENT failed."
+
+  (let* ((parse-failed? #f)
+         (alist (map
+                 (match-lambda
+                   ((file-name . content)
+                    (let ((tree failed?
+                                (string->parse-tree content
+                                                    #:file-name file-name
+                                                    #:content-alist alist)))
+                      (when failed?
+                        (set! parse-failed? #t))
+                      (cons file-name tree))))
+                 alist)))
+    (values alist parse-failed?)))
 
 (define* (parse-tree-alist->ast alist #:key (content-alist '())
                                 (working-directory (getcwd)))
@@ -210,7 +259,6 @@ parse CONTENT and return
      (IMPORTED-FILE-NAME . IMPORTED-PARSE-TREE) ...)
 
 using CONTENT-ALIST to transform locations."
-
   (define (expand-imports ast-alist)
     (let ((file (car ast-alist))
           (imports (cdr ast-alist)))
@@ -242,32 +290,40 @@ using CONTENT-ALIST to transform locations."
   "Parse FILE-NAME and return an ast.  When PARSE-TREE?, return the
 parse trees.  When SKIP-WFC?, skip the well-formedness checks.  Unless
 @var{debug?}, handle exceptions."
-
   (define (helper)
     (if (equal? file-name "-") (string->ast (read-string)
                                             #:parse-tree? parse-tree?
                                             #:skip-wfc? skip-wfc?
                                             #:transform transform)
-        (let* ((content-alist dir (file+import-content-alist file-name #:imports imports))
-               (parse-tree-alist (parse-file+import-content-alist content-alist)))
-          (if parse-tree? parse-tree-alist
+        (let* ((content-alist
+                dir
+                (file+import-content-alist file-name #:imports imports))
+               (parse-tree-alist
+                parse-failed?
+                (parse-file+import-content-alist content-alist))
+               (skip-ast? (or parse-tree? (null? parse-tree-alist))))
+          (if skip-ast? (values parse-tree-alist parse-failed?)
               (let* ((ast (parse-tree-alist->ast parse-tree-alist
                                                  #:content-alist content-alist
                                                  #:working-directory dir))
+                     (ast (silence:annotate-functions ast))
                      (ast (if skip-wfc? ast
                               (ast:wfc ast)))
                      (transform (map string->transformation transform)))
-                ((apply compose identity (reverse transform)) ast))))))
+                (values ((apply compose identity (reverse transform)) ast)
+                        parse-failed?))))))
 
   (catch (if debug? 'none #t)
     helper
-    (parse:handle-exceptions file-name)))
+    (lambda (key . args)
+      (apply (parse:handle-exceptions file-name) key args)
+      (values #f #t))))
 
 (define (parse:handle-exceptions file-name)
   (lambda (key . args)
     (case key
       ((syntax-error)
-       (exit EXIT_FAILURE))
+       #t)
       ((import-error)
        (match args
          ((file imports content-alist)
@@ -282,41 +338,43 @@ parse trees.  When SKIP-WFC?, skip the well-formedness checks.  Unless
                    (let loop ((from from))
                      (when from
                        (format (current-error-port) "imported from ~a\n" from)
-                       (loop (assoc-ref imported-from (basename from))))))))))
-       (exit EXIT_FAILURE))
+                       (loop (assoc-ref imported-from (basename from)))))))))))
       ((error)
-       (apply format (current-error-port) "~a:error:~a\n" file-name args)
-       (exit EXIT_FAILURE))
+       (apply format (current-error-port) "~a:error:~a\n" file-name args))
       ((well-formedness-error)
-       (for-each wfc:report-message args)
-       (exit EXIT_FAILURE))
+       (for-each wfc:report-message args))
       ((system-error)
        (let ((errno (system-error-errno (cons key args))))
          (format (current-error-port) "~a: ~a\n"
-                 (strerror errno) file-name))
-       (exit EXIT_FAILURE))
-      (else (format (current-error-port) "internal error: ~a: ~a: ~s\n"
-                    file-name key args)
-            (exit EXIT_FAILURE)))))
+                 (strerror errno) file-name)))
+      (else
+       (format (current-error-port) "internal error: ~a: ~a: ~s\n"
+               file-name key args)))))
 
 (define* (call-with-handle-exceptions thunk #:key backtrace? (file-name "-"))
   (catch (if backtrace? 'none #t)
     thunk
-    (parse:handle-exceptions file-name)))
+    (lambda (key . args)
+      (apply (parse:handle-exceptions file-name) key args)
+      (exit EXIT_FAILURE))))
 
 (define* (string->ast string #:key parse-tree? skip-wfc? (transform '()))
   "Parse STRING and return an ast.  When PARSE-TREE?, return the parse
 trees.  When SKIP-WFC? skip the well-formedness checks."
   (let* ((content-alist dir (string->file+import-content-alist string))
-         (parse-tree-alist (parse-file+import-content-alist content-alist)))
-    (if parse-tree? parse-tree-alist
+         (parse-tree-alist parse-failed? (parse-file+import-content-alist
+                                          content-alist))
+         (skip-ast? (or parse-tree? (null? parse-tree-alist))))
+    (if skip-ast? (values parse-tree-alist parse-failed?)
         (let* ((ast (parse-tree-alist->ast parse-tree-alist
                                            #:content-alist content-alist
                                            #:working-directory dir))
+               (ast (silence:annotate-functions ast))
                (ast (if skip-wfc? ast
                         (ast:wfc ast)))
                (transform (map string->transformation transform)))
-          ((apply compose identity (reverse transform)) ast)))))
+          (values ((apply compose identity (reverse transform)) ast)
+                  parse-failed?)))))
 
 (define (string->file+import-content-alist string)
   "Split possibly pre-processed STRING at preprocessing markers
@@ -354,35 +412,30 @@ and the working directory.
                                           (string-match imported-regexp string))))))))
           (values alist dir)))))
 
-(define* (file->stream file-name #:key debug? (imports '()))
+(define* (file->stream file-name #:key (imports '()))
   "Read @var{file-name}, using @var{imports} to resolve @code{import}
 statements and return the expanded dezyne text, similar to @command{gcc
--E}.  Unless @var{debug?}, handle exceptions."
+-E}."
 
-  (define (helper)
-    (let* ((content-alist
-            dir
-            (if (equal? file-name "-") (string->file+import-content-alist (read-string))
-                (file+import-content-alist file-name #:imports imports)))
-           (file (car content-alist))
-           (imports (cdr content-alist)))
-      (string-join
-       (append
-        (match file
-          ((file-name . content)
-           (if (string-prefix? "#dir " content) (list content)
-               (list (format #f "#dir ~s" dir)
-                     (format #f "#file ~s" file-name)
-                     content))))
-        (append-map (match-lambda ((file-name . content)
-                                   (list (format #f "#imported ~s" file-name)
-                                         content)))
-                    imports))
-       "\n")))
-
-  (catch (if debug? 'none #t)
-    helper
-    (parse:handle-exceptions file-name)))
+  (let* ((content-alist
+          dir
+          (if (equal? file-name "-") (string->file+import-content-alist (read-string))
+              (file+import-content-alist file-name #:imports imports)))
+         (file (car content-alist))
+         (imports (cdr content-alist)))
+    (string-join
+     (append
+      (match file
+        ((file-name . content)
+         (if (string-prefix? "#dir " content) (list content)
+             (list (format #f "#dir ~s" dir)
+                   (format #f "#file ~s" file-name)
+                   content))))
+      (append-map (match-lambda ((file-name . content)
+                                 (list (format #f "#imported ~s" file-name)
+                                       content)))
+                  imports))
+     "\n")))
 
 (define* (file+import-content-alist file-name #:key (imports '()) (content-alist '()))
   "Recursively resolve imports starting with FILE-NAME and return two
