@@ -25,6 +25,7 @@
 (define-module (dzn trace)
   #:use-module (ice-9 match)
   #:use-module (ice-9 pretty-print)
+  #:use-module (ice-9 regex)
 
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9 gnu)
@@ -42,15 +43,15 @@
 (define (trace-parse ascii)
   (define (-sexp- str len pos)
     (and (< pos len) (eq? #\( (string-ref str pos))
-     (catch #t
-       (lambda ()
-         (with-input-from-string (substring str pos len)
-           (lambda () (let* ((sexp (read))
-                             (end (+ pos (ftell (current-input-port)))))
-                        (list end (substring str pos end))))))
-       (lambda (key . args)
-         (warn 'sexp-parse-failed: key args)
-         #f))))
+         (catch #t
+           (lambda ()
+             (with-input-from-string (substring str pos len)
+               (lambda () (let* ((sexp (read))
+                                 (end (+ pos (ftell (current-input-port)))))
+                            (list end (substring str pos end))))))
+           (lambda (key . args)
+             (warn 'sexp-parse-failed: key args)
+             #f))))
   (define-peg-pattern sexp all -sexp-)
 
   (define-peg-string-patterns
@@ -466,19 +467,26 @@ ws               <   [ \t]
        (q-instance? (communication-left o))))
 
 (define (step->event o)
-  (cond ((communication? o)
-         (let ((instance (cond ((and=> (communication-left o) external-instance?) (communication-left o))
-                               ((and=> (communication-right o) external-instance?) (communication-right o))
-                               ((and=> (communication-left o) q-instance?) (communication-right o))
-                               (else (if (communication-left o) (communication-left o)
-                                         (communication-right o))))))
-           (string-join
-            (append
-             (cdr instance)
-             (list (communication-event o)))
-            ".")))
-        ((message? o)
-         (message->string o))))
+  (cond
+   ((communication? o)
+    (let ((instance
+           (cond
+            ((and=> (communication-left o) q-instance?)
+             (communication-right o))
+            ((and=> (communication-left o) external-instance?)
+             (communication-left o))
+            ((and=> (communication-right o) external-instance?)
+             (communication-right o))
+            ((communication-left o))
+            (else
+             (communication-right o)))))
+      (string-join
+       (append
+        (cdr instance)
+        (list (communication-event o)))
+       ".")))
+   ((message? o)
+    (message->string o))))
 
 (define (serialize o)
   (let ((x (record->alist o)))
@@ -702,8 +710,8 @@ ws               <   [ \t]
                                 (string-append
                                  (location-prefix communication #:from? #t)
                                  (substring life-line 0 (max 0 (1+ right-margin)))
-                                event
-                                (substring life-line (+ right-margin 2)))
+                                 event
+                                 (substring life-line (+ right-margin 2)))
                                 (string-append
                                  (location-prefix communication #:from? #f)
                                  (substring life-line 0 (max 0 (1+ right-margin)))
@@ -889,7 +897,7 @@ ws               <   [ \t]
               (header     (make-lifeline-header name kind))
               (prefix     (string-append name "."))
               (labels     (cond ((or (member kind '(component interface)))
-                                 '("<back>"))
+                                 '("<back>" "<defer>"))
                                 ((and (eq? kind 'provides)
                                       (pair? labels)
                                       (not (string-index (car labels) #\.)))
@@ -1029,18 +1037,28 @@ ws               <   [ \t]
                 activities
                 (append events (list event)))))))))
 
-(define (steps->trail steps)
-  (let ((steps (filter (conjoin (negate q-out?) external?) steps)))
+(define* (steps->trail steps #:key meta?)
+  (define (defer-qout? step)
+    (and (message? step)
+         (equal? (message-message step) "<defer>")))
+  (define (meta-event? message)
+    (let ((event (message-message message)))
+      (or (string-suffix? "<flush>" event)
+          (string-match "^<[-a-z]+>$" event))))
+  (let ((steps (filter (disjoin (conjoin (negate q-out?) external?)
+                                defer-qout?
+                                (conjoin (const meta?) message? meta-event?))
+                       steps)))
     (map step->event steps)))
 
-(define* (trace:format-trace trace #:key debug? file-name format internal? locations?)
+(define* (trace:format-trace trace #:key debug? file-name format internal? locations? meta?)
   (let* ((structured (trace:trace->structured trace #:file-name file-name #:debug? debug?))
          (merged (merge-communications structured)))
     (cond
      ((equal? format "sexp")
       (map serialize structured))
      ((equal? format "event")
-      (let ((steps (steps->trail merged)))
+      (let ((steps (steps->trail merged #:meta? meta?)))
         (string-join steps "\n" 'suffix)))
      ((equal? format "diagram")
       (let* ((header (find header? structured))
