@@ -33,7 +33,7 @@
 
   #:use-module (dzn ast goops)
   #:use-module (dzn ast)
-  #:use-module (dzn code dzn)
+  #:use-module (dzn code language dzn)
   #:use-module (dzn misc)
   #:use-module (dzn trace)
   #:use-module (dzn vm ast)
@@ -46,9 +46,13 @@
             display-trace-n
             display-trails
             set-trigger-locations
+            strip-sut-prefix
             trace-equal?
+            trace-name
             trace->trail
+            trace->component-trail
             trace->string-trail
+            trace->component-string-trail
             report))
 
 ;;;
@@ -76,10 +80,21 @@
 (define-method (trace->trail (o <program-counter>))
   (and (pc-event? o) (pc->event o)))
 
+(define (strip-sut-prefix o)
+  (if (string-prefix? "sut." o) (substring o 4) o))
+
 (define (trace->string-trail trace)
   (let ((trail (map cdr (trace->trail trace))))
-    (define (strip-sut-prefix o)
-      (if (string-prefix? "sut." o) (substring o 4) o))
+    (map strip-sut-prefix trail)))
+
+(define-method (trace->component-trail (o <list>))
+  (filter-map trace->component-trail (reverse o)))
+
+(define-method (trace->component-trail (o <program-counter>))
+  (and (pc-arrow? o) (pc->component-event o)))
+
+(define (trace->component-string-trail trace)
+  (let ((trail (map cdr (trace->component-trail trace))))
     (map strip-sut-prefix trail)))
 
 (define (trace-equal? a b)
@@ -137,6 +152,12 @@
 (define-method (pc-event? (o <runtime:component>) (trigger <q-trigger>))
   #f)
 
+(define-method (pc-event? (o <runtime:port>) (q-in <q-in>))
+  #t)
+
+(define-method (pc-event? (o <runtime:component>) (q-in <q-in>))
+  #t)
+
 (define-method (pc-event? (o <runtime:port>) (action <action>))
   (or (ast:in? action)
       (is-a? (%sut) <runtime:port>)
@@ -153,8 +174,7 @@
 (define-method (pc-event? (o <runtime:component>) (q-out <q-out>))
   (let* ((trigger (.trigger q-out))
          (port (.port trigger)))
-    (or (ast:provides? port)
-        (ast:external? port))))
+    (ast:provides? port)))
 
 (define-method (pc-event? (pc <program-counter>) (o <trigger-return>))
   (and (not (is-a? (.event (.trigger pc)) <modeling-event>))
@@ -251,6 +271,14 @@
 (define-method (pc->event (o <runtime:component>) (defer-qout <defer-qout>))
   (cons defer-qout "<defer>"))
 
+(define-method (pc->event (o <runtime:component>) (q-in <q-in>))
+  (let* ((trigger (.trigger q-in))
+         (port (.port trigger))
+         (r:port (runtime:port o port))
+         (r:other-port (and r:port (runtime:other-port r:port)))
+         (trigger (trigger->component-trigger r:other-port trigger)))
+    (cons q-in (trigger->string trigger))))
+
 (define-method (pc->event (o <runtime:component>) (q-out <q-out>))
   (let ((trigger (.trigger q-out)))
     (cons q-out (trigger->string trigger))))
@@ -279,6 +307,59 @@
 (define-method (pc->event x y)
   #f)
 
+;;; component-events formatting
+
+(define-method (pc->component-event (o <program-counter>))
+  (if (and=> (.status o) (negate (is? <end-of-trail>))) (pc->component-event (.status o))
+      (pc->component-event o (.statement o))))
+
+(define-method (pc->component-event (pc <program-counter>) (o <statement>))
+  (pc->component-event (.instance pc) o))
+
+(define-method (pc->component-event (pc <program-counter>) (o <initial-compound>))
+  (pc->component-event (.instance pc) o (.trigger pc)))
+
+(define-method (pc->component-event (o <runtime:port>) (compound <initial-compound>) (trigger <trigger>))
+  (cons trigger
+        (if (is-a? (%sut) <runtime:port>)
+            (format #f "~a" (.event.name trigger))
+            (format #f "~a.~a" (.name (.ast o)) (.event.name trigger)))))
+
+(define-method (pc->component-event (o <runtime:component>) (compound <initial-compound>) (trigger <trigger>))
+  (cons trigger
+        (let* ((port (.port trigger))
+               (r:port (runtime:port o port)))
+          (format #f "~a.~a" (trace-name r:port) (.event.name trigger)))))
+
+(define-method (pc->component-event (o <runtime:component>) (action <action>))
+  (cons action
+        (let* ((port (.port action))
+               (r:port (runtime:port o port)))
+          (cond
+           ((ast:injected? port)
+            (format #f "~a.~a" (trace-name r:port) (.event.name action)))
+           (else
+            (format #f "~a.~a" (trace-name r:port) (.event.name action)))))))
+
+(define-method (pc->component-event (pc <program-counter>) (o <trigger-return>))
+  (pc->component-event (.instance pc) (clone o #:event.name (format #f "~a" (.event.name o)))))
+
+(define-method (pc->component-event (o <runtime:component>) (return <trigger-return>))
+  (cons return
+        (let ((port (.port return)))
+          (let* ((r:port (runtime:port o port))
+                 (r:other-port (and r:port (runtime:other-port r:port))))
+            (if (eq? r:port r:other-port) ;The other port is injected
+                (format #f "~a.~a" (trace-name r:port) (.event.name return))
+                (let ((value (.event.name return)))
+                  (format #f "~a.~a" (trace-name r:port) value)))))))
+
+(define-method (pc->component-event x)
+  (pc->event x))
+
+(define-method (pc->component-event x y)
+  (pc->event x y))
+
 
 ;;;
 ;;; Trace (a.k.a. arrows, broken arrows)
@@ -314,9 +395,6 @@
 (define-method (pc-arrow? (o <runtime:component>) (trigger <q-trigger>))
   #f)
 
-(define-method (pc-arrow? (o <runtime:component>) (q-in <q-in>))
-  #t)
-
 (define-method (pc-arrow? (o <runtime:port>) (action <action>))
   (or (ast:in? action)
       (is-a? (%sut) <runtime:port>)
@@ -347,7 +425,7 @@
                (not (ast:modeling? trigger)))))))
 
 (define-method (pc-arrow? (o <runtime:component>) (return <trigger-return>))
-  #t)
+  (.port return))
 
 (define-method (pc-arrow? x y)
   (pc-event? x y))
@@ -421,6 +499,11 @@
   (cons defer-qout
         (format #f "<defer>")))
 
+(define-method (pc->arrow (o <runtime:port>) (q-in <q-in>))
+  (cons q-in
+        (let ((trigger (.trigger q-in)))
+          (format #f "... <- ~a.~a" (runtime:instance->string o) (.event.name trigger)))))
+
 (define-method (pc->arrow (o <runtime:component>) (q-in <q-in>))
   (cons q-in (format #f "~a.<q> <- ..." (runtime:instance->string o))))
 
@@ -555,7 +638,7 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
   (define (injected-port-name component interface)
     (let* ((ports (filter ast:injected? (ast:port* component)))
            (port (find (compose (cute ast:eq? <> interface) .type) ports)))
-      (.name port)))
+      (and=> port .name)))
 
   (let loop ((trace trace) (result '()))
     (if (null? trace) (reverse result)
@@ -570,7 +653,7 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                  ((is-status? <compliance-error>) (last result))
                  (is-a? statement <action>)
                  (not (.component-acceptance (.status (last result))))
-                 (.port-acceptance  (.status (last result))))
+                 (.port-acceptance (.status (last result))))
             (let* ((last-pc (last result))
                    (status (.status last-pc))
                    (r:port (.port status))
@@ -614,7 +697,7 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                    (other-interface (.type other-port))
                    (action (clone action #:parent other-interface))
                    (action-pc (clone pc #:instance r:other-port #:statement action)))
-              (loop (cdr trace) (cons*  pc action-pc result))))
+              (loop (cdr trace) (cons* pc action-pc result))))
            ((and next
                  (is-a? statement <action>)
                  (ast:out? statement)
@@ -650,6 +733,19 @@ Add (synthesize) missing PCs for <q-in>, <q-out> and <trigger-return>."
                                      #:instance r:other-port
                                      #:statement action)))
               (loop (cdr trace) (cons* pc action-pc result))))
+           ((and (is-a? statement <q-in>)
+                 (is-a? pc-instance <runtime:component>))
+            (let* ((trigger (.trigger statement))
+                   (port (.port trigger))
+                   (r:port (runtime:port pc-instance port))
+                   (r:other-port (and r:port (runtime:other-port r:port)))
+                   (other-port (.ast r:other-port))
+                   (other-interface (.type other-port))
+                   (trigger (clone trigger #:port.name #f))
+                   (qin (clone statement #:trigger trigger))
+                   (qin (clone qin #:parent other-interface))
+                   (qin-pc (clone pc #:instance r:other-port #:statement qin)))
+              (loop (cdr trace) (cons* qin-pc pc result))))
            ((and (is-a? statement <q-out>)
                  (is-a? pc-instance <runtime:component>))
             (let* ((trigger (.trigger statement))
@@ -793,7 +889,7 @@ intermediate steps such as assignments, function calls, replies,
          (string-join
           (map (cut format #f "~aerror: ~a\n" <> message) locations) "")))
       (($ <queue-full-error>)
-       (let* ((model ((compose  ast:dotted-name .type .ast .instance) status))
+       (let* ((model ((compose ast:dotted-name .type .ast .instance) status))
               (location (location-prefix (.ast status))))
          (format #f "~aerror: queue-full in component ~s\n" location model)))
       (($ <range-error>)
