@@ -74,6 +74,7 @@
             get-variables
             graft-locals
             in-event?
+            instance-rtc-trigger
             is-status?
             label->string
             label?
@@ -118,7 +119,6 @@
             set-handling!
             set-reply
             set-state
-            set-variables
             show-eligible
             switch-context
             string->q-trigger
@@ -128,9 +128,11 @@
             string->value
             trace-head:eq?
             trigger->component-trigger
+            trigger-in-q?
             trigger->system-trigger
             trigger->port-trigger
             trigger->string
+            update-other-state
             update-state)
   #:re-export (eval-expression))
 
@@ -184,9 +186,9 @@
                                   (negate
                                    (cute equal? <> "<defer>")))))
                         trail))
-         (loop  (and loop-index (call-with-values
-                                    (cute split-at trail loop-index)
-                                  (lambda (a b) b))))
+         (loop (and loop-index (call-with-values
+                                   (cute split-at trail loop-index)
+                                 (lambda (a b) b))))
          (trail (if loop (append trail loop loop)
                     trail)))
     trail))
@@ -287,7 +289,7 @@ under certain conditions; type `,show c' for details.
           %package-name))
 
 (define %warranty-info
-  (format #f  "~a is distributed WITHOUT ANY WARRANTY.  The following sections
+  (format #f "~a is distributed WITHOUT ANY WARRANTY.  The following sections
 from the GNU General Public License, version 3, should make that clear.
 
   15. Disclaimer of Warranty.
@@ -423,6 +425,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
      (string-append (ast:name (.type.name o)) ":" (.field o)))
     (($ <literal>)
      (label->string (.value o)))
+    (($ <q-out>)
+     (trigger->string (.trigger o)))
     ((? (is? <trigger>))
      (trigger->string o))
     (($ <trigger-return>)
@@ -650,6 +654,14 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (rtc-port (pc <program-counter>))
   (and=> (rtc-trigger pc) .port))
 
+(define-method (instance-rtc-trigger (pc <program-counter>))
+  (let* ((instance (.instance pc))
+         (pcs (pc->stack pc))
+         (pcs (filter (compose (cute eq? <> instance) .instance) pcs))
+         (triggers (filter-map .trigger pcs)))
+    (and (pair? triggers)
+         (last triggers))))
+
 
 ;;;
 ;;; Blocking
@@ -874,12 +886,12 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 ;;; Q and flush
 ;;;
 
-(define-method (enqueue (pc <program-counter>)  (ast <ast>) (instance <runtime:component>) (trigger <trigger>))
+(define-method (enqueue (pc <program-counter>) (ast <ast>) (instance <runtime:component>) (trigger <trigger>))
   (%debug "  ~s ~s ~a => ~s\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) "<enqueue>" (trigger->string trigger))
   (let* ((state (get-state pc instance))
          (q (.q state)))
     (if (= (length q) (%queue-size))
-        (let ((error (make <queue-full-error> #:ast ast  #:instance instance
+        (let ((error (make <queue-full-error> #:ast ast #:instance instance
                            #:message "queue-full")))
           (clone pc #:status error))
         (set-deferred (set-state pc (clone state #:q (append q (list trigger)))) instance))))
@@ -963,7 +975,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (flush (pc <program-counter>) instance)
   (%debug "  ~s ~s ~a\n" (name instance) (and=> (.trigger pc) trigger->string) "<flush>")
   (let* ((orig-pc pc)
-         (flush-return (make <flush-return>))
+         (trigger (and=> (as (.q (get-state pc instance)) <pair>)
+                         car))
+         (flush-return (make <flush-return> #:trigger trigger))
+         (flush-return (clone flush-return #:parent (and=> trigger .parent)))
          (pc (push-pc pc flush-return))
          (pc (clone pc #:instance instance)))
     (cond
@@ -999,11 +1014,54 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (get-state o (.instance o)))
 
 (define-method (set-state (pc <program-counter>) (o <state>))
-  (clone pc #:state (clone (.state pc) #:state-list (map (lambda (x) (if (eq? (.instance o) (.instance x)) o x)) ((compose .state-list .state) pc)))))
+  (define (replace-instance-state x)
+    (if (eq? (.instance o) (.instance x)) o
+        x))
+  (let* ((state-list ((compose .state-list .state) pc))
+         (state-list (map replace-instance-state state-list))
+         (state (clone (.state pc) #:state-list state-list)))
+    (clone pc #:state state)))
 
 (define-method (set-state (pc <program-counter>) (state <list>))
+
+  (define (assign state variable value)
+    (let ((name (.name variable)))
+      (or (range-error variable value)
+          (clone state #:variables (assoc-set! (copy-tree (.variables state)) name value)))))
+
+  (define (update-variable update-list variable state)
+    (let ((name (string->symbol (.name variable))))
+      (or (and=> (assoc-ref update-list name) (compose (cut assign state variable <>) sexp->value))
+          state)))
+
+  (define (update-state event state pc)
+    (let* ((instance (.instance state))
+           (path (map string->symbol (runtime:instance->path instance)))
+           (update-list (assoc-ref event path))
+           (result (fold (cut update-variable update-list <> <>) state ((compose ast:variable* .type .ast) instance))))
+      (if (is-a? result <error>) (clone pc #:status result)
+          (set-state pc result))))
+
   (fold (cut update-state state <> <>) pc ((compose .state-list .state) pc)))
 
+(define* (update-other-state pc instance #:key (direction? ast:provides?))
+  (let ((other-instance (runtime:other-port instance)))
+    (cond
+     ((not other-instance)
+      pc)
+     ((direction? instance)
+      (let ((variables (get-variables pc instance)))
+        (set-variables pc other-instance variables)))
+     (else
+      (let* ((variables (get-variables pc other-instance)))
+        (set-variables pc instance variables))))))
+
+(define (update-state pc from-instance from-pc)
+  (let* ((from-state (get-state from-pc from-instance))
+         (pc (set-state pc from-state)))
+    (if (is-a? (%sut) <runtime:component>) pc
+        (update-other-state pc from-instance))))
+
 (define-method (get-reply (pc <program-counter>) (instance <runtime:instance>) (port <string>))
   (assoc-ref (.reply (get-state pc instance)) port))
 
@@ -1042,7 +1100,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (.variables (get-state pc instance)))
 
 (define-method (set-variables (pc <program-counter>) (o <list>))
-  (set-state pc (clone (get-state pc) #:variables o)))
+  (set-state pc (clone (get-state pc) #:variables (copy-tree o))))
+
+(define-method (set-variables (pc <program-counter>) (instance <runtime:instance>) (o <list>))
+  (set-state pc (clone (get-state pc instance) #:variables (copy-tree o))))
 
 (define-method (get-members (pc <program-counter>) instance)
   (let* ((state (get-state pc instance))
@@ -1074,12 +1135,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (pop-locals (pc <program-counter>) (o <list>))
   (set-variables pc (drop (get-variables pc) (length o))))
 
-(define-method (eval-expression (o <state>) (e <expression>))
-  (eval-expression (.variables o) e))
-
-(define-method (eval-expression (pc <program-counter>) (e <expression>))
-  (eval-expression (get-state pc) e))
-
 (define-method (range-error o (value <expression>))
   (unless (or (is-a? o <formal>) (is-a? o <variable>))
     (error "range-error" o))
@@ -1098,7 +1153,7 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (assign (pc <program-counter>) variable expression)
   (let* ((name (.name variable))
          (state (get-state pc))
-         (value (eval-expression state expression))
+         (value (eval-expression pc expression))
          (state (clone state #:variables (assoc-set! (copy-tree (.variables state)) name value)))
          (pc (set-state pc state))
          (error (range-error variable value)))
@@ -1133,10 +1188,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 (define-method (assign (pc <program-counter>) (variable <formal>) (e <call>))
   (assign pc variable (.return pc)))
 
-(define (rewrite-trace-head rewriter trace)
+(define (rewrite-trace-head rewriter trace . rest)
   (match trace
     ((pc tail ...)
-     (cons (rewriter pc) tail))))
+     (cons (apply rewriter pc rest) tail))))
 
 (define-method (append-port-trace (pc <program-counter>) trace (port-instance <runtime:port>) port-trace)
   (let* ((ipc (car port-trace))
@@ -1157,13 +1212,61 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
     (lambda _
       (serialize o (current-output-port)))))
 
+(define* ((merge-port-state state-list) component)
+
+  (define (variables-prefixed-with-name port)
+    (map (match-lambda
+           ((variable . value)
+            (cons (string-append ((compose .name .ast .instance) port)
+                                 "."
+                                 variable) value)))
+         (.variables port)))
+
+  (let ((ports (filter (compose (cute eq? (.instance component) <>)
+                                .container .instance)
+                       state-list)))
+    (clone component
+           #:variables (append
+                        (append-map variables-prefixed-with-name ports)
+                        (.variables component)))))
+
 (define-method (serialize (o <system-state>) port)
-  (display "(state " port)
-  (for-each (lambda (x) (unless (eq? x ((compose car .state-list) o))
-                          (display " " port))
-                    (serialize x port))
-            (.state-list o))
-  (display ")" port))
+  (let* ((state-list (.state-list o))
+         (provides-boundary (filter (compose
+                                     (conjoin
+                                      runtime:boundary-port?
+                                      (compose ast:provides? .ast))
+                                     .instance)
+                                    state-list))
+         (requires-boundary (filter (compose
+                                     (conjoin
+                                      runtime:boundary-port?
+                                      (compose ast:requires? .ast))
+                                     .instance)
+                                    state-list))
+         (components (filter (compose (is? <runtime:component>)
+                                      .instance)
+                             state-list))
+         (components (map (merge-port-state state-list) components))
+         (interface (if (pair? components) '()
+                        (filter (compose (is? <runtime:port>)
+                                         .instance)
+                                state-list)))
+         (state-list (append interface
+                             provides-boundary
+                             components
+                             requires-boundary)))
+    (define (serialize-state o)
+      (unless (eq? o (car state-list))
+        (display " " port))
+      (serialize o port))
+    (cond
+     ((null? state-list)
+      (display "(state)" port))
+     (else
+      (display "(state " port)
+      (for-each serialize-state state-list)
+      (display ")" port)))))
 
 (define-method (serialize (o <state>) port)
   (let ((path ((compose runtime:instance->path .instance) o)))
@@ -1179,20 +1282,17 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
       (display `("*q*" . ,(map trigger->string (.q o))) port))
     (display ")" port)))
 
-(define-method (serialize-header (o <system-state>))
-  (with-output-to-string
-    (lambda _ (cons 'header (serialize-header o (current-output-port))))))
-
 (define-method (serialize-header (o <system-state>) port)
-  (display "(header " port)
-  (for-each (lambda (x)
-              (unless (eq? x ((compose car %instances)))
-                (display " " port))
-              (serialize-header x port))
-            (filter (disjoin (negate (is? <runtime:port>))
-                             runtime:boundary-port?)
-                    (%instances)))
-  (display ")" port))
+  (let ((instances (filter (disjoin (negate (is? <runtime:port>))
+                                    runtime:boundary-port?)
+                           (%instances))))
+    (define (display-instance o)
+      (unless (eq? o (car instances))
+        (display " " port))
+      (serialize-header o port))
+    (display "(header " port)
+    (for-each display-instance instances)
+    (display ")" port)))
 
 (define-method (serialize-header (o <runtime:instance>) port)
   (display "(" port)
@@ -1218,20 +1318,6 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
             (make <enum-literal> #:type.name (make <scope.name> #:ids ids) #:field field))))) ;; FIXME: what about resolving
     ))
 
-(define (update-variable update-list variable pc)
-  (let ((name (string->symbol (.name variable))))
-    (or (and=> (assoc-ref update-list name)
-               (compose (cute assign pc variable <>) sexp->value))
-        pc)))
-
-(define (update-state event state pc)
-  (let* ((instance (.instance state))
-         (path (map string->symbol (runtime:instance->path instance)))
-         (update-list (assoc-ref event path))
-         (pc (clone pc #:instance instance))
-         (variables ((compose ast:variable* .type .ast) instance)))
-    (fold (cute update-variable update-list <> <>) pc variables)))
-
 
 ;;;
 ;;; Hashing state
@@ -1425,9 +1511,24 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (and (requires-trigger? event)
        (find (match-lambda
                ((port trigger tail ...)
-                (equal? (trigger->string trigger) event)))
+                (let ((trigger (trigger->component-trigger port trigger)))
+                  (equal? event (trigger->string trigger)))))
              (.external-q pc))))
 
+(define-method (trigger-in-q? (pc <program-counter>) event)
+  (and (external-trigger? event)
+       (let* ((trigger (string->trigger event))
+              (r:port (runtime:port-name->instance (.port.name trigger)))
+              (r:component-port (runtime:other-port r:port))
+              (instance (.container r:component-port))
+              (state (get-state pc instance))
+              (q (.q state)))
+         (match q
+           ((trigger rest ...)
+            (let ((trigger (trigger->component-trigger r:port trigger)))
+              (equal? event (trigger->string trigger))))
+           (() #f)))))
+
 (define-method (return-trigger? event)
   (and (string? event)
        (let ((event (match (string-split event #\.)
@@ -1534,9 +1635,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
 ;;;
 
 (define-method (init (o <variable>))
-  (let ((value (eval-expression '() (.expression o))))
+  (let* ((pc (make <program-counter>))
+         (value (eval-expression pc (.expression o))))
     (or (range-error o value)
-        (cons (.name o) (eval-expression '() (.expression o))))))
+        (cons (.name o) (eval-expression pc (.expression o))))))
 
 (define-method (make-state (o <runtime:instance>))
   (let* ((variables (map init ((compose ast:variable* .type .ast) o)))
@@ -1545,7 +1647,14 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
         (make <state> #:instance o #:variables variables))))
 
 (define-method (make-system-state instances)
-  (make <system-state> #:state-list (map make-state (filter (disjoin runtime:boundary-port? (is? <runtime:component>)) instances))))
+  (let* ((instances
+          (filter (disjoin
+                   (is? <runtime:component>)
+                   (if (is-a? (%sut) <runtime:system>) (is? <runtime:port>)
+                       runtime:boundary-port?))
+                  instances))
+         (lst (map make-state instances)))
+    (make <system-state> #:state-list lst)))
 
 (define* (make-pc #:key (instances (%instances)) (trail '()))
   (let* ((system-state (make-system-state instances))
