@@ -24,28 +24,16 @@
 #include <dzn/locator.hh>
 #include <dzn/runtime.hh>
 
-namespace library {
+namespace library
+{
 
-  foreign::foreign(const dzn::locator& dzn_locator)
-  : dzn_meta{"","foreign",0,{},{},{[this]{w.check_bindings();}}}
-  , dzn_rt(dzn_locator.get<dzn::runtime>())
-  , dzn_locator(dzn_locator)
-  , w({{"w",&w,this,&dzn_meta},{"",0,0,0}})
-  {
-    w.in.world = [&](){return dzn::call_in(this,[=]{
-      w_world();
-      this->dzn_rt.flush(this);
-    }, this->w, "world");};
-  }
-  void foreign::w_world()
-  {
-  }
-  void foreign::check_bindings() const
-  {
-    dzn::check_bindings(&dzn_meta);
-  }
-  void foreign::dump_tree(std::ostream& os) const
-  {
-    dzn::dump_tree(os, &dzn_meta);
-  }
+foreign::foreign (const dzn::locator &dzn_locator)
+  : dzn_meta{"", "foreign", 0, {}, {}, {[this]{w.dzn_check_bindings ();}}}
+, dzn_runtime (dzn_locator.get<dzn::runtime> ())
+, dzn_locator (dzn_locator)
+, w ({{"w", &w, this, &dzn_meta}, {"", 0, 0, 0}})
+{
+  w.in.world = [&] {};
+  w.in.world.set (this, &w, "world");
+}
 };
